
// CaptureRawSDIDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CaptureRawSDI.h"
#include "CaptureRawSDIDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CCaptureRawSDIDlg dialog
CCaptureRawSDIDlg::CCaptureRawSDIDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCaptureRawSDIDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CCaptureRawSDIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST, m_List);
}

BEGIN_MESSAGE_MAP(CCaptureRawSDIDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_START, &CCaptureRawSDIDlg::OnBnClickedStart)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CCaptureRawSDIDlg message handlers
bool FillDeviceList(CComboBox& list);
BOOL CCaptureRawSDIDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	if( !FillDeviceList(m_List) ) {
		MessageBox(_T("Can't detect any capture source!\nSelect mode 'Monitor' for any FDExt board."), _T("ERROR on Caprure Raw SDI Start"), MB_ICONEXCLAMATION);
		EndDialog(IDCANCEL);
	}

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CCaptureRawSDIDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCaptureRawSDIDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
// !!!!!!!!!!!!!!!!!!!!!!!!! SDK USAGE CODE HERE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
#include "FDExtSrvd.h"
#include "FDExtSrvd_i.c"
//
const int cMaxFrameCnt = 50*1; // 1 seconds
//
bool FillDeviceList(CComboBox& list)
{
	// create FDExt manager
	CComPtr<IFDExtMgr> pIMgr;
	HRESULT hres = pIMgr.CoCreateInstance(CLSID_FDExtMgr);
	if( hres != S_OK )  return false;
	ATLASSERT(pIMgr != NULL);
	// find RawSDI capture objects
	long cnt=0;
	hres = pIMgr->get_FDObjectsCount(&cnt);
	ATLASSERT(S_OK==hres);
	if( cnt < 1 )  return false;
	int nCaptureCnt = 0;
	for( long n=0; n<cnt; n++ ) {
		CFDObjectInfo info;
		HRESULT hres = pIMgr->get_FDObjectInfoByIndex(n,&info);
		ATLASSERT(S_OK==hres);
		if( FAILED(hres) )  continue;
		if( info.m_FDObjType==FDObjType_RawSDISrc && info.m_Valid>0 ) { // use the capturer
			CString name;
			name.Format(_T("%s %s"), info.m_BoardName, info.m_FDObjRegName);
			list.AddString(name);
			nCaptureCnt++;
		}
	}
	list.SetCurSel(0);
	return nCaptureCnt > 0;
}
//
HRESULT RunCapture(CString szDevName, BYTE *pFrame[cMaxFrameCnt], ULONG nSize[cMaxFrameCnt], int &step)
{
	CComPtr<IFDCaptureRawSDI> pICapturer;
	// create FDExt manager
	CComPtr<IFDExtMgr> pIMgr;
	HRESULT hres = pIMgr.CoCreateInstance(CLSID_FDExtMgr);
	step = 1;
	if( hres != S_OK )  return hres;
	ATLASSERT(pIMgr != NULL);
	// find capture object with name 'szDevName'
	long cnt=0;
	hres = pIMgr->get_FDObjectsCount(&cnt);
	ATLASSERT(S_OK==hres);
	step = 2;
	if( cnt < 1 )  return E_UNEXPECTED;
	step = 3;
	for( long n=0; n<cnt; n++ ) {
		CFDObjectInfo info;
		HRESULT hres = pIMgr->get_FDObjectInfoByIndex(n,&info);
		ATLASSERT(S_OK==hres);
		if( FAILED(hres) )  continue;
		if( info.m_FDObjType==FDObjType_RawSDISrc && info.m_Valid>0 ) {
			CString name;
			name.Format(_T("%s %s"), info.m_BoardName, info.m_FDObjRegName);
			if( name != szDevName )  continue;
			// use the capture object
			CComPtr<IUnknown> pIUnk;
			hres = pIMgr->get_FDObjectByIndex(n,FDObjType_RawSDISrc,&pIUnk);
			ATLASSERT(S_OK==hres);
			step = 4;
			if( hres != S_OK )  return hres;
			hres = pIUnk->QueryInterface(&pICapturer);
			step = 5;
			if( hres != S_OK )  return hres;
			break;
		}
	}
	if( pICapturer==NULL )  return E_UNEXPECTED;
	// obtain raw frame size
	ULONG size=0,dx=0,dy=0;  CRawSDIDataType nDataType=CRawSDIDataType_HD;  BOOLEAN bProgressive=FALSE;
	hres = pICapturer->FDGetSDIMaxDataParams(&size,&dx,&dy);
	ATLASSERT(hres==S_OK);
	step = 6;
	if( hres != S_OK )  return hres;
	// allocate memory
	for( int i=0; i<cMaxFrameCnt; i++ ) {
		pFrame[i] = new BYTE[size];
		if( pFrame[i]==NULL ) {
			step = 7;
			return E_OUTOFMEMORY;
		}
	}
	// start capturing
	ULONGLONG nFrame=0,n=0;
	hres = pICapturer->FDGetSDITime(&nFrame,&n);
	if( hres==S_FALSE ) {
		Sleep(250);
		hres = pICapturer->FDGetSDITime(&nFrame,&n);
	}
	step = 8;
	if( hres != S_OK )  return hres;
	for( int i=0; i<cMaxFrameCnt; ) {
		// try capture frame
		n = nFrame;
		hres = pICapturer->FDReadSDIData(pFrame[i],size,&nSize[i],&n,&nDataType,&dx,&dy,&bProgressive);
		if( hres==S_FALSE ) { // no data - wait more
			Sleep(10);
			continue;
		}
		ATLASSERT(nDataType==CRawSDIDataType_HD || nDataType==CRawSDIDataType_SD || nDataType==CRawSDIDataType_ASI);
		step = 9;
		if( hres != S_OK )  return hres;
		step = 10;
		if( n > nFrame )  return E_FAIL;
		// all right - go to next frame
		nFrame++;
		i++;
	}
	// ???
	step = 999;
	return S_OK;
}
//
HRESULT RunSave(BYTE *pFrame[cMaxFrameCnt], ULONG nSize[cMaxFrameCnt], int &step)
{
	step = 1000;
	// obtain file name
	SYSTEMTIME st;
	GetSystemTime(&st);
	for( int i=0; i<cMaxFrameCnt; i++ ) {
		// create output file
		CString fname;
		fname.Format(_T("RawSDI_%d%02d%02d_%02d%02d%02d_%03d.raw"), st.wYear,st.wMonth,st.wDay, st.wHour,st.wMinute,st.wSecond, i);
		HANDLE hFile = CreateFile(fname,GENERIC_WRITE,0,NULL,CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL);
		step = 2000+i;
		if( hFile==INVALID_HANDLE_VALUE )  return HRESULT_FROM_WIN32(GetLastError());
		// write data
		DWORD n = 0;
		step = 3000+i;
		if( !WriteFile(hFile, pFrame[i],nSize[i],&n, NULL) )  return HRESULT_FROM_WIN32(GetLastError());
		step = 4000+i;
		if( n < nSize[i] )  return E_FAIL;
		step = 5000+i;
		if( !CloseHandle(hFile) )  return HRESULT_FROM_WIN32(GetLastError());
	}
	step = 999;
	return S_OK;
}
//
int ScanVBILines(BYTE *pData10, ULONG nSize10)
{
	ULONG nSize16 = nSize10 * 4 / 5;
	WORD *pData16 = new WORD[nSize16];
	if( pData16==NULL )  return 0;
	// convert raw data (10-bit) to UYVY (16-bit per U, U, or V with mask 0x3FFF)
	BYTE *pi = pData10;
	WORD *po = pData16;
	WORD *poe = pData16 + nSize16;
	while (po < poe) {
		ULONG	v	=	*((ULONG*)pi);
		po[0] = (v >> 00) & 0x3ff;
		po[1] = (v >> 10) & 0x3ff;
		po[2] = (v >> 20) & 0x3ff;
		po[3] = ((v >> 30) & 0x3) + (USHORT(pi[4]) << 2);
		po += 4;
		pi += 5;
	}
	// scan VBI from HD-SDI
	const int cDX = 2640; // raw HD-SDI line width (pixels)
	const int cX0 = 2640-1920; // VBI data at the end of line
	const int cVBI = 1920;
	int mask = 0;
	for( int nLine=0; nLine<25; nLine++ ) {
		WORD *ptr = &pData16[cDX*2 * nLine];
		// skip non-VBI data
		ptr += cX0 * 2;
		for( int i=0; i<cVBI; i++ ) {
			if( ptr[0] != (0x80<<2)  ||  ptr[1] != (0x10<<2) ) {
				mask |= 1 << nLine;
				break;
			}
			ptr += 2;
		}
	}
	return mask;
}
//
//
void CCaptureRawSDIDlg::OnBnClickedStart()
{
	BYTE *pFrame[cMaxFrameCnt]={0};   ULONG nSize[cMaxFrameCnt]={0};
	CString szDevName;
	m_List.GetLBText(m_List.GetCurSel(), szDevName);
	int step = 0;
	HCURSOR hCursor = ::SetCursor(::LoadCursor(NULL,IDC_WAIT));
	HRESULT hres = RunCapture(szDevName, pFrame,nSize,step);
	if( hres != S_OK ) {
		::SetCursor(hCursor);
		CString str;  str.Format(_T("Capture error on step #%d, hres=0x%X"),step,hres);
		MessageBox(str,_T("Caprure Raw SDI"), MB_ICONEXCLAMATION);
		return;
	}
	step = 0;
	hres = RunSave(pFrame,nSize,step);
	if( hres != S_OK ) {
		::SetCursor(hCursor);
		CString str;  str.Format(_T("Save error on step #%d, hres=0x%X"),step,hres);
		MessageBox(str,_T("Capture Raw SDI"), MB_ICONEXCLAMATION);
		return;
	}
	int mask = ScanVBILines(pFrame[0], nSize[0]);
	for( int i=0; i<cMaxFrameCnt; i++ ) if( pFrame[i] != NULL )  delete[] pFrame[i];
	Sleep(1000);
	::SetCursor(hCursor);
	CString str;  str.Format(_T("All right.\n\nVBI_mask = 0x%X"), mask);
	MessageBox(str, _T("Capture Raw SDI"), MB_OK);
}
